
/**
 * @file        kmdw_ddr_mem_mgr.h
 * @brief       ddr memory malloc/free APIs
 *
 * @copyright   Copyright (c) 2021 Kneron Inc. All rights reserved.
 */

#ifndef KMDW_DDR_MEM_MGR_H
#define KMDW_DDR_MEM_MGR_H

#include "com.h"

/**
 * @brief application client ID, used a tag attached to every allocated DDR block
 */
typedef enum {
    FW_INFO_DDR,       /**< DDR block for fw_info */
    MODEL_DDR,         /**< DDR block for model */
    CAM_IMG_DDR,       /**< DDR block for image from camera */
    FID_DDR,           /**< DDR block for FID application(app) */
    DB_DDR,            /**< DDR block for DB(app) */
    SI_DDR,            /**< DDR block for SI mode */
    ISI_DDR,           /**< DDR block for ISI mode */
    FD_AGE_GENDER_DDR, /**< DDR block for fd/age_gender (app) */
    DME_DDR,           /**< DDR block for DME mode */
    JPEG_ENC_DDR,      /**< DDR block for jpeg encode */
    JPEG_DEC_DDR,      /**< DDR block for jpeg decode */
    HICO_DDR,          /**< DDR block for HICO mode */
    TOF_DEC_DDR,       /**< DDR block for tof decode */
    TOTAL_DDR_CLIENTS, /**< dummy item */
} ddr_client_id_t;

/**
 * @brief to allocate DDR memory aligned (64 bytes alignment) for the specified client
 * @param id: application client ID
 * @param numbtye: size in byte
 * @return the address of allocated block
 */
uint32_t kmdw_ddr_malloc(ddr_client_id_t id, uint32_t numbyte);

/**
 * @brief free the specified DDR block for the specified client
 * @param id: application client ID
 * @param mem_addr: memory address
 */
void kmdw_ddr_free(ddr_client_id_t id, uint32_t mem_addr);

/**
 * @brief free all DDR blocks associated with the specified client
 * @param id: application client ID
 */
void kmdw_ddr_clean_client(ddr_client_id_t id);

/**
 * @brief inquery if the specified client has at least one associated DDR block
 * @param id: application client ID
 * @return TURE: has one assocated block at least; FALSE: none
 */
bool kmdw_ddr_is_mem_available(ddr_client_id_t id);

#endif

